import itertools
import logging
import multiprocessing
import numpy
import pathfinder.model.oscillating


def print_result(msg, result):
	logging.info(msg)
	logging.info(f"\tResult: {result.pathfinder_x}")
	logging.info(f"\tSuccess: {result.success}. {result.message}")
	try:
		logging.info(f"\tFunc evals: {result.nfev}")
	except AttributeError:
		pass
	try:
		logging.info(f"\tJacb evals: {result.njev}")
	except AttributeError:
		pass


def main():
	logging.info("Running script...")

	dot_inputs = [
		([0, 0, .01], 5), ([-1, 0, -.01], 5), ([-2, 0, -.01], 5), ([0, -1, .01], 5), ([-1, -1, 0], 5), ([-2, -1, 0], 5),
		([0, 0, .01], 1), ([-1, 0, -.01], 1), ([-2, 0, -.01], 1), ([0, -1, .01], 1), ([-1, -1, 0], 1), ([-2, -1, 0], 1),
	]
	dipole = pathfinder.model.oscillating.OscillatingDipole([1, 2, 3], [0, 4, -1], 7)
	expected_result = numpy.array([1, 2, 3, 0, 4, -1, 7])
	dipole_arrangement = pathfinder.model.oscillating.OscillatingDipoleArrangement([dipole])
	dot_measurements = dipole_arrangement.get_dot_measurements(dot_inputs)

	model = pathfinder.model.oscillating.DotOscillatingDipoleModel(dot_measurements, 1)
	res = model.sol_basinhopping()

	logging.info("Result:")
	logging.info(res)

if __name__ == "__main__":
	logging.basicConfig(level=logging.DEBUG)
	main()
