import itertools
import logging
import multiprocessing
import numpy
import pathfinder.model.oscillating


def print_result(msg, result):
	logging.info(msg)
	logging.info(f"\tResult: {result.pathfinder_x}")
	logging.info(f"\tSuccess: {result.success}. {result.message}")
	try:
		logging.info(f"\tFunc evals: {result.nfev}")
	except AttributeError:
		pass
	try:
		logging.info(f"\tJacb evals: {result.njev}")
	except AttributeError:
		pass


def try_initial_position(model, expected_result, initial_pos):
	res = model.sol(initial_position=initial_pos)
	if res.success:
		return res.pathfinder_x
	else:
		return None


def main():
	logging.info("Running script...")
	dot_inputs = list(itertools.chain.from_iterable(
		(([0 + o, 0, .01], f), ([0 + o, -1, 0], f), ([-1 + o, 0, -.01], f), ([-1 + o, -1, .01], f)) for f in numpy.arange(1, 10, .05) for o in (0, 0.5)
	))

	dipole = pathfinder.model.oscillating.OscillatingDipole([1, 2, 3], [0, 0, -1], 7)
	expected_result = numpy.array([1, 2, 3, 0, 0, -1, 7])
	dipole_arrangement = pathfinder.model.oscillating.OscillatingDipoleArrangement([dipole])
	dot_measurements = dipole_arrangement.get_dot_measurements(dot_inputs)

	model = pathfinder.model.oscillating.DotOscillatingDipoleModel(dot_measurements, 1)
	logging.info("Finished setting up model")

	vals = [
		[1.61794, 1.96704, -4.00528, -0.0784033, 0.316984, 1.0257, 7],
		[8.18934, 2.86237, 15.2491, -0.695465, 1.11861, -0.306364, 7],
		[0.7485, -2.17157, 35.6347, 0.353949, 0.487858, 0.0554771, 7],
		[2.11867, -0.227147, -48.9911, -0.180317, 0.521937, 0.00989036, 7],
		[0.0930268, 3.32614, 1.69929, 0.121074, -0.230094, -0.872652, 7],
		[1, 2, 3, 0, 0, -1, 7],
		[1, 2, 3, 0, 0, -1, 8],
		[2, 2, 3, 0, 0, -1, 7]
	]
	jac = model.jac()
	costs = model.costs()
	for val in vals:
		j = jac(numpy.array(val))
		c = costs(numpy.array(val))
		logging.info(f"looking at {val}")
		# logging.info(f"jac: [{j}]")
		# logging.info(f"cost: [{c}]")
		logging.info(f"gradient: {numpy.matmul(numpy.transpose(j), c)}")

if __name__ == "__main__":
	logging.basicConfig(level=logging.DEBUG)
	main()
