import numpy
import pathfinder.model.oscillating


def chunk_n_sort(pts):
	pt_length = 7
	chunked_pts = [pts[i: i + pt_length] for i in range(0, len(pts), pt_length)]

	return chunked_pts


def print_result(msg, result):
	print(msg)
	print(f"\tResult: {result.pathfinder_x}")
	print(f"\tSuccess: {result.success}. {result.message}")
	try:
		print(f"\tFunc evals: {result.nfev}")
	except AttributeError:
		pass
	try:
		print(f"\tJacb evals: {result.njev}")
	except AttributeError:
		pass


def test_simple_three_dipole_one_dot_frequencies():
	# setup
	dot_inputs = [
		([0, 0, .01], f) for f in numpy.arange(1, 10, .5)
	]
	dipole = pathfinder.model.oscillating.OscillatingDipole([1, 2, 3], [0, 4, -1], 7)
	dipole2 = pathfinder.model.oscillating.OscillatingDipole([2, 5, 0], [1, 6, 1], 2)
	dipole3 = pathfinder.model.oscillating.OscillatingDipole([-1, 2, 0], [-1, 2, 1], 4)

	dipole_arrangement = pathfinder.model.oscillating.OscillatingDipoleArrangement([dipole, dipole2, dipole3])
	dot_measurements = dipole_arrangement.get_dot_measurements(dot_inputs)

	model = pathfinder.model.oscillating.DotOscillatingDipoleModel(dot_measurements, 3)
	res = model.simple_sol()

	print_result("six oscillating dipole one dot", res)
	assert res.success, "The solution for a single dipole and six dots should have succeeded."
	# numpy.testing.assert_allclose(res.pathfinder_x, expected_result, err_msg="Dipole wasn't as expected.", rtol=1e-6, atol=1e-6)
